"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cdn = void 0;
const cdn_client_1 = require("tencentcloud-sdk-nodejs/tencentcloud/services/cdn/v20180606/cdn_client");
const formatter_1 = require("./formatter");
const capi_1 = require("@tencent-sdk/capi");
const common_1 = require("@tencent-serverless-framework/common");
const ONE_SECOND = 1000;
const TIMEOUT = 5 * 60 * ONE_SECOND;
/** 腾讯云内容分发网络 (Cdn)for Serverless */
class Cdn {
    /**
     * 构造函数
     *
     * @param credentials 认证信息
     */
    constructor(credentials = {}) {
        this.credentials = credentials;
        this.client = new cdn_client_1.Client({
            credential: common_1.removeUndefined({
                secretId: credentials.SecretId,
                secretKey: credentials.SecretKey,
                token: credentials.Token,
            }),
            region: 'ap-guangzhou',
            profile: {},
        });
        this.capi = new capi_1.Capi({
            ServiceType: common_1.ServiceType.cdn,
            Region: 'ap-guangzhou',
            SecretId: this.credentials.SecretId,
            SecretKey: this.credentials.SecretKey,
            Token: this.credentials.Token,
        });
    }
    /**
     * purgeCdnCache 用于批量提交目录刷新，根据域名的加速区域进行对应区域的刷新。
     * 内部调用 PurgePathCache 接口
     *
     * @param urls 要刷新的路径
     * @param flushType 刷新方式：flush：刷新产生更新的资源 delete：刷新全部资源
     */
    async purgeCdnUrls(urls, flushType = 'flush') {
        console.log(`Purging CDN caches, it will work in 5 minutes...`);
        try {
            await this.client.PurgePathCache({
                Paths: urls,
                FlushType: flushType,
            });
        }
        catch (e) {
            console.error(e.message);
            console.error(e);
        }
    }
    // FIXME: userAgent 默认应为 TencentCDN
    /**
     * pushCdnUrls 用于将指定 URL 资源列表加载至 CDN 节点，支持指定加速区域预热。
     * 内部调用 PushUrlsCache 接口
     *
     * @param urls URL 列表，需要包含协议头部 http:// 或 https://
     * @param userAgent 指定预热请求回源时 HTTP 请求的 User-Agent 头部，默认为 flush
     * @param area 预热生效区域 mainland：预热至境内节点 overseas：预热至境外节点 global：预热全球节点 不填充情况下，默认为 mainland， URL 中域名必须在对应区域启用了加速服务才能提交对应区域的预热任务
     */
    async pushCdnUrls(urls, userAgent = 'flush', area = 'mainland') {
        console.log(`Pushing CDN caches...`);
        try {
            await this.client.PushUrlsCache({
                Urls: urls,
                Area: area,
                UserAgent: userAgent,
            });
        }
        catch (e) {
            // no op
        }
    }
    /**
     * 根据域名查询 CDN 信息
     * 内部调用 DescribeDomains
     *
     * @param domain 域名
     * @returns 第一个找到的 CDN 信息
     */
    async getCdnByDomain(domain) {
        const { Domains } = await this.client.DescribeDomains({
            Filters: [{ Name: 'domain', Value: [domain] }],
        });
        if (Domains && Domains.length) {
            return Domains[0];
        }
        return undefined;
    }
    /**
     * 下线指定域名的 CDN 服务
     *
     * @param domain 域名
     */
    async offlineCdnDomain(domain) {
        var _a;
        const Status = (_a = (await this.getCdnByDomain(domain))) === null || _a === void 0 ? void 0 : _a.Status;
        if (Status === 'online') {
            // disable first
            await this.client.StopCdnDomain({ Domain: domain });
        }
        else if (Status === 'processing') {
            throw new Error(`Status is not operational for ${domain}, status: ${Status}`);
        }
    }
    /**
     * 启用用户的 CDN 服务
     * (相当于在控制台点击开启 CDN 服务)
     */
    async openCdnService() {
        try {
            await this.capi.request({
                Action: 'OpenCdnService',
                PayTypeMainland: 'flux',
                PayTypeOverseas: 'flux',
            }, {
                isV3: false,
                debug: false,
                RequestClient: 'TencentCloud - ServerlessComponentCdn',
                host: `${common_1.ServiceType.cdn}.tencentcloudapi.com`,
                path: '/',
            });
        }
        catch (e) {
            if (e.code !== 'ResourceInUse.CdnUserExists') {
                throw e;
            }
        }
    }
    /**
     * 部署 CDN
     *
     * @param inputs 部署 CDN 时的输入
     * @returns 返回部署 CDN 后的输出
     */
    async deploy(inputs) {
        var _a, _b;
        const inputStr = JSON.stringify(inputs);
        await this.openCdnService();
        /** 仅刷新，不进行部署 */
        if (inputs.onlyRefresh === true) {
            const domainExist = await this.getCdnByDomain(inputs.domain);
            // refresh cdn urls
            if (domainExist && inputs.refreshCdn && inputs.refreshCdn.urls) {
                await this.purgeCdnUrls(inputs.refreshCdn.urls, inputs.refreshCdn.flushType);
            }
            const outputs = {
                resourceId: domainExist === null || domainExist === void 0 ? void 0 : domainExist.ResourceId,
                https: !!inputs.https,
                domain: inputs.domain,
                origins: inputs.origin && inputs.origin.origins,
                cname: `${inputs.domain}.cdn.dnsv1.com`,
                refreshUrls: (_a = inputs.refreshCdn) === null || _a === void 0 ? void 0 : _a.urls,
            };
            return outputs;
        }
        const sourceInputs = JSON.parse(JSON.stringify(inputs));
        const origin = formatter_1.formatter.originInputs(inputs.origin);
        const outputs = {
            https: !!inputs.https,
            domain: inputs.domain,
            origins: origin.Origins,
            cname: `${inputs.domain}.cdn.dnsv1.com`,
            inputCache: inputStr,
        };
        const cdnInfo = await this.getCdnByDomain(inputs.domain);
        /** 如果 inputs 和 inputCache 相同，则不再进行部署 */
        const inputCache = (_b = inputs.oldState) === null || _b === void 0 ? void 0 : _b.inputCache;
        if (inputCache && inputCache === JSON.stringify(sourceInputs)) {
            console.log(`No configuration changes for CDN domain ${inputs.domain}`);
            outputs.resourceId = cdnInfo === null || cdnInfo === void 0 ? void 0 : cdnInfo.ResourceId;
        }
        else {
            const res = await this.createOrUpdateCdn(inputs);
            outputs.resourceId = res.resourceId;
        }
        return outputs;
    }
    /**
     * 创建或更新 CDN
     *
     * @param inputs 输入
     * @returns 结果
     */
    async createOrUpdateCdn(inputs) {
        var _a, _b;
        const outputs = {};
        let cdnInfo = await this.getCdnByDomain(inputs.domain);
        /** 如果 CDN 已下线，先删除再创建 */
        // FIXME: 传入 serviceType 会导致错误，故不一样时删除后重建
        if (cdnInfo && cdnInfo.Status === 'offline' && cdnInfo.ServiceType !== inputs.serviceType) {
            console.log(`The CDN domain ${inputs.domain} is offline.`);
            console.log(`Recreating CDN domain ${inputs.domain}`);
            await this.client.DeleteCdnDomain({ Domain: inputs.domain });
            cdnInfo = undefined;
        }
        /** 如果 CDN 存在且未关闭，更新 */
        if (cdnInfo) {
            // update
            console.log(`The CDN domain ${inputs.domain} has existed.`);
            console.log('Updating...');
            // TODO: when update, VIP user can not set ServiceType parameter, need CDN api optimize
            // if (inputs.serviceType && String(inputs.serviceType) !== cdnInfo.ServiceType) {
            //   inputs.serviceType = cdnInfo.ServiceType!;
            // }
            const cdnInputs = formatter_1.formatter.deployInputs(inputs);
            // FIXME: 传入 serviceType 会导致错误
            delete cdnInputs.ServiceType;
            await this.client.UpdateDomainConfig(cdnInputs);
            outputs.resourceId = cdnInfo.ResourceId;
        }
        else {
            /** CDN 不存在，创建 CDN */
            console.log(`Adding CDN domain ${inputs.domain}...`);
            try {
                // if not config ServiceType, default to web
                inputs.serviceType = (_a = inputs.serviceType) !== null && _a !== void 0 ? _a : 'web';
                const cdnInputs = formatter_1.formatter.deployInputs(inputs);
                await this.client.AddCdnDomain(cdnInputs);
            }
            catch (e) {
                if (e.code === 'ResourceNotFound.CdnUserNotExists') {
                    console.log(`Please goto https://console.cloud.tencent.com/cdn open CDN service.`);
                }
                throw e;
            }
            await common_1.sleep(1000);
            const detail = await this.getCdnByDomain(inputs.domain);
            outputs.resourceId = detail === null || detail === void 0 ? void 0 : detail.ResourceId;
        }
        // 是否等待 CDN 部署完毕
        // When set syncFlow false, just continue, do not wait for online status.
        if (inputs.async === false) {
            console.log('Waiting for CDN deploy success, it maybe cost 5 minutes....');
            await common_1.waitResponse({
                callback: async () => this.getCdnByDomain(inputs.domain),
                targetProp: 'Status',
                targetResponse: 'online',
                timeout: TIMEOUT,
            });
            // FIXME: 应当先刷新（purge），再预热（push）
            // push cdn urls
            if ((_b = inputs.pushCdn) === null || _b === void 0 ? void 0 : _b.urls) {
                await this.pushCdnUrls(inputs.pushCdn.urls, inputs.pushCdn.area, inputs.pushCdn.userAgent);
            }
            // refresh cdn urls
            if (inputs.refreshCdn) {
                await this.purgeCdnUrls(inputs.refreshCdn.urls);
            }
        }
        console.log(`CDN deploy success to domain: ${inputs.domain}`);
        return outputs;
    }
    /**
     * 删除指定域名的 CDN
     *
     * @param inputs 输入信息
     * @param inputs.domain 输入中的域名
     * @returns 成功后返回空 object
     */
    async remove(inputs) {
        const { domain } = inputs;
        if (!domain) {
            throw new common_1.ApiTypeError(`PARAMETER_CDN`, 'domain is required');
        }
        // need circle for deleting, after domain status is 6, then we can delete it
        console.log(`Start removing CDN for ${domain}`);
        const detail = await this.getCdnByDomain(domain);
        if (!detail) {
            console.log(`CDN domain ${domain} not exist`);
            return {};
        }
        const { Status } = detail;
        if (Status === 'online') {
            // disable first
            await this.client.StopCdnDomain({ Domain: domain });
        }
        else if (Status === 'processing') {
            console.log(`Status is not operational for ${domain}`);
            return {};
        }
        console.log(`Waiting for offline ${domain}...`);
        await common_1.waitResponse({
            callback: async () => this.getCdnByDomain(domain),
            targetProp: 'Status',
            targetResponse: 'offline',
            timeout: TIMEOUT,
        });
        console.log(`Removing CDN for ${domain}`);
        await this.client.DeleteCdnDomain({ Domain: domain });
        console.log(`Removed CDN for ${domain}.`);
        return {};
    }
}
exports.Cdn = Cdn;
